unit FileDrop;

interface

uses Windows,ActiveX,Classes;
type
    TFileDropEvent=procedure(Sender:TObject;const FileList:TStringList)of object;
    TFileAcceptEvent=procedure(Sender:TObject;const FileList:TStringList;var CanAccept:boolean) of object;
    TFileDropAcceptor=class(TInterfacedObject,IDropTarget)
   private
      FFileList:TStringList;
      FOnFilesDropped:TFileDropEvent;
      FOnFileAccept:TFileAcceptEvent;
   public
      constructor Create(AOnDrop:TFileDropEvent;AOnEnter:TFileAcceptEvent);
      destructor Destroy;override;
      function DragEnter(const dataObj:IDataObject;grfKeyState:Longint;pt:TPoint;var dwEffect:Longint):HResult;stdcall;
      function DragOver(grfKeyState:Longint;pt:TPoint;var dwEffect:Longint):HResult;stdcall;
      function DragLeave:HResult;stdcall;
      function Drop(const dataObj:IDataObject;grfKeyState:Longint;pt:TPoint;var dwEffect:Longint):HResult;stdcall;
      property OnFilesDropped:TFileDropEvent
        read FOnFilesDropped write FOnFilesDropped;
   end;

implementation
uses ShellAPI;

constructor TFileDropAcceptor.Create(AOnDrop:TFileDropEvent;AOnEnter:TFileAcceptEvent);
begin
     inherited Create;
     FFileList:=TStringList.Create;
     FOnFilesDropped:=AOnDrop;
     FOnFileAccept:=AOnEnter;
end;

destructor TFileDropAcceptor.Destroy;
begin
     FFileList.Free;
     inherited Destroy;
end;

function TFileDropAcceptor.DragEnter(const dataObj:IDataObject;grfKeyState:Longint;pt:TPoint;var dwEffect:Longint):HResult;stdcall;
var Medium:TSTGMedium;
    Format:TFormatETC;
    NumFiles:Integer;
    i:Integer;
    rslt:Integer;
    FileName:array [0..MAX_PATH]of char;
    S:string;
    CanDrop:boolean;
begin
     dataObj._AddRef;
     Format.cfFormat:=CF_HDROP;
     Format.ptd:=Nil;
     Format.dwAspect:=DVASPECT_CONTENT;
     Format.lindex:=-1;
     Format.tymed:=TYMED_HGLOBAL;
     rslt:=dataObj.GetData(Format,Medium);
     FFileList.Clear;
     if rslt=S_OK then 
     begin
      NumFiles:=DragQueryFile(Medium.hGlobal,$FFFFFFFF,nil,0);
      for i:=0 to NumFiles-1 do
      begin
        DragQueryFile(Medium.hGlobal,i,FileName,sizeof(FileName));
        S:=FileName;
        FFileList.Add(S);
      end;
     end;
     if Medium.unkForRelease=nil then ReleaseStgMedium(Medium);
     dataObj._Release;
     CanDrop:=False;
     if FFileList.Count>0 then 
     begin
       CanDrop:=True;
       if Assigned(FOnFileAccept) then FOnFileAccept(Self,FFileList,CanDrop);
     end;
     Result:=S_OK;
end;

function TFileDropAcceptor.DragOver(grfKeyState:Longint;pt:TPoint;var dwEffect:Longint):HResult;stdcall;
begin
     if FFileList.Count>0 then dwEffect:=DROPEFFECT_COPY
     else dwEffect:=DROPEFFECT_NONE;
     Result:=S_OK;
end;

function TFileDropAcceptor.DragLeave:HResult;stdcall;
begin
     Result:=S_OK;
end;

function TFileDropAcceptor.Drop(const dataObj:IDataObject;grfKeyState:Longint;pt:TPoint;var dwEffect:Longint):HResult;stdcall;
begin
     if Assigned(FOnFilesDropped) and (FFileList.Count>0) then 
     begin
       FOnFilesDropped(Self,FFileList);
       dwEffect:=DROPEFFECT_COPY;
     end
     else dwEffect:=DROPEFFECT_NONE;
     Result:=S_OK;
end;

initialization
   OleInitialize(nil);

finalization
   OleUninitialize;

end.
